package com.docus.server.report.thread;

import com.docus.core.util.DateUtil;
import com.docus.core.util.Func;
import com.docus.server.report.config.JobAdminConfig;
import com.docus.server.report.consts.ReportDownloadWait;
import com.docus.server.report.entity.DelayedMessages;
import com.docus.server.report.listener.NisReportDownloadWaitHandler;
import com.docus.server.report.mapper.DelayedMessagesMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * @author YongBin Wen
 * @date 2024/1/23 10:59
 */

public class VerifyNisReportHelper {
    private static Logger logger = LoggerFactory.getLogger(VerifyNisReportHelper.class);
    private static VerifyNisReportHelper instance = new VerifyNisReportHelper();

    public static VerifyNisReportHelper getInstance() {
        return instance;
    }

    private Thread nisVerifyThread;
    private volatile boolean toStop = false;

    public void start() {
        nisVerifyThread = new Thread(() -> {
            final String messageType = ReportDownloadWait.NIS_REPORT_DOWNLOAD_WAIT_DELAY_MSG_TYPE;
            DelayedMessages condition = new DelayedMessages();
            condition.setMessageType(messageType);
            DelayedMessagesMapper delayedMessagesMapper = JobAdminConfig.getJobAdminConfig().getDelayedMessagesMapper();
            NisReportDownloadWaitHandler handler = JobAdminConfig.getJobAdminConfig().getNisReportDownloadWaitHandler();
            while (!toStop) {
                try {
                    LocalDateTime currentDateTime = LocalDateTime.now();
                    String formatCurrentDateTime = Func.formatDateTime(currentDateTime);
                    // 数据量暂时不大，使用
                    List<DelayedMessages> delayedMessages = delayedMessagesMapper.findExecutableMessages(condition, formatCurrentDateTime);
                    if (Func.isNotEmpty(delayedMessages)) {
                        for (DelayedMessages message : delayedMessages) {
                            try {
                                String patientId = message.getMessage();
                                handler.verify(patientId);
                                delayedMessagesMapper.delById(message.getId());
                            } catch (Exception ex) {
                                if (!toStop) {
                                    logger.error(">>>>>>>>>>> verify nis report thread error:" + ex.getMessage() + " ,delayMessage:" + Func.toJson(message), ex);
                                    int retries = message.getRetries() + 1;
                                    Integer delaySeconds = message.getDelaySeconds();
                                    LocalDateTime nexExecuteLocalDateTime = LocalDateTime.now().plusSeconds(delaySeconds);
                                    String dateTime = Func.formatDateTime(nexExecuteLocalDateTime);
                                    Date nexExecuteTime = Func.parseDate(dateTime, DateUtil.PATTERN_DATETIME);

                                    message.setRetries(retries);
                                    message.setErrorMessage(ex.getMessage());
                                    message.setNextExecutionTime(nexExecuteTime);
                                    delayedMessagesMapper.updateById(message);
                                }
                            }
                        }
                    }

                    TimeUnit.SECONDS.sleep(10);
                } catch (Exception ex) {
                    if (!toStop) {
                        logger.error(">>>>>>>>>>> verify nis report thread error:" + ex.getMessage(), ex);
                    }
                }

            }
            logger.info(">>>>>>>>>>>  verify nis report thread stop");
        });
        nisVerifyThread.setDaemon(true);
        nisVerifyThread.setName("verifyNisReportHelper");
        nisVerifyThread.start();
    }

    public void toStop() {
        toStop = false;
        nisVerifyThread.interrupt();
        try {
            nisVerifyThread.join();
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
        }
    }
}
